package com.example.PhidgetHelloWorldExample;

import android.app.Activity;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;
import android.content.Context;

import com.phidget22.*;

import java.util.ArrayList;

public class HelloWorldExample extends Activity {

	Manager manager;

	ArrayList<Phidget> listItems=new ArrayList<Phidget>();
	ArrayAdapter<Phidget> adapter;

	Toast errToast;

	/** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);

        try
        {
        	manager = new Manager();

			//Allow direct USB connection of Phidgets
        	if(getPackageManager().hasSystemFeature(PackageManager.FEATURE_USB_HOST))
				com.phidget22.usb.Manager.Initialize(this);

			//Enable server discovery to list remote Phidgets
			this.getSystemService(Context.NSD_SERVICE);
			Net.enableServerDiscovery(ServerType.DEVICE_REMOTE);

			manager.addAttachListener(new ManagerAttachListener() {
				public void onAttach(final ManagerAttachEvent attachEvent) {
				    AttachEventHandler handler = new AttachEventHandler(attachEvent);
                   synchronized(handler)
					{
						runOnUiThread(handler);
						try {
							handler.wait();
						} catch (InterruptedException e) {
							e.printStackTrace();
						}
					}
				}
			});

			manager.addDetachListener(new ManagerDetachListener() {
				public void onDetach(final ManagerDetachEvent detachEvent) {
                    DetachEventHandler handler = new DetachEventHandler(detachEvent);
                    synchronized(handler)
					{
						runOnUiThread(handler);
						try {
							handler.wait();
						} catch (InterruptedException e) {
							e.printStackTrace();
						}
					}
				}
			});

			//set up an array adapter to display all connected Phidget channels
			adapter = new ArrayAdapter<Phidget>(this, android.R.layout.simple_list_item_2, android.R.id.text1, listItems) {
				@Override
				public View getView(int position, View convertView, ViewGroup parent) {
					View view = super.getView(position, convertView, parent);
					TextView text1 = (TextView) view.findViewById(android.R.id.text1);
					TextView text2 = (TextView) view.findViewById(android.R.id.text2);

					try {
						text1.setText(listItems.get(position).getDeviceName());
						text2.setText(listItems.get(position).getChannelName() +
								"\nSerial: " + String.valueOf(listItems.get(position).getDeviceSerialNumber()) +
								"\nChannel: " + String.valueOf(listItems.get(position).getChannel()) +
								"      Hub Port: " + String.valueOf(listItems.get(position).getHubPort()) +
								"\nLabel: " + listItems.get(position).getDeviceLabel());
					} catch (PhidgetException pe) {
						pe.printStackTrace();
					}

					return view;
				}
			};

			((ListView)findViewById(R.id.deviceList)).setAdapter(adapter);

			manager.open();
        } catch (PhidgetException pe) {
	        pe.printStackTrace();
		}

    }
	
    class AttachEventHandler implements Runnable {
        ManagerAttachEvent ev;

		public AttachEventHandler(ManagerAttachEvent ev) {
			this.ev = ev;
		}

		public void run() {

			Phidget ch = ev.getChannel();
			listItems.add(0, ch);
			adapter.notifyDataSetChanged();

			//notify that we're done
			synchronized(this)
			{
				this.notify();
			}
		}
    }
    
    class DetachEventHandler implements Runnable {
        ManagerDetachEvent ev;
    	
    	public DetachEventHandler(ManagerDetachEvent ev) {
    		this.ev = ev;
    	}
    	
		public void run() {
			Phidget ch = ev.getChannel();
			listItems.remove(ch);
			adapter.notifyDataSetChanged();

			//notify that we're done
			synchronized(this)
			{
				this.notify();
			}
		}
    }

	class ErrorEventHandler implements Runnable {
		Phidget ch;
		ErrorEvent errorEvent;

		public ErrorEventHandler(Phidget ch, ErrorEvent errorEvent) {
			this.ch = ch;
			this.errorEvent = errorEvent;
		}

		public void run() {
			 if (errToast == null)
				 errToast = Toast.makeText(getApplicationContext(), errorEvent.getDescription(), Toast.LENGTH_SHORT);

			 //replace the previous toast message if a new error occurs
			 errToast.setText(errorEvent.getDescription());
			 errToast.show();
        }
	}

    @Override
    protected void onDestroy() {
    	super.onDestroy();
    	try {
			manager.close();

		} catch (PhidgetException e) {
			e.printStackTrace();
		}

		//Disable USB connection to Phidgets
		if(getPackageManager().hasSystemFeature(PackageManager.FEATURE_USB_HOST))
            com.phidget22.usb.Manager.Uninitialize();
    }

}

